use core::slice;

/// VGA driver for mode 0x13:
///
/// - mode: graphics
/// - text res: 40x25
/// - pixel box: 8x8
/// - pixel res: 320x200
/// - colors: 256/256k
/// - addr: A000
/// - pixel format: RRRGGGBB
pub struct Vga13h {
	// Double frame buffers
	fb_a: [u8; Vga13h::WIDTH * Vga13h::HEIGHT],
	fb_b: [u8; Vga13h::WIDTH * Vga13h::HEIGHT],

	/// If true, show fb_a (and write to fb_b).
	/// if false, show fb_b.
	show_fb_a: bool,
}

impl Vga13h {
	pub const WIDTH: usize = 320;
	pub const HEIGHT: usize = 200;
	pub const ADDR: usize = 0xA0000;

	pub const unsafe fn new() -> Self {
		Self {
			fb_a: [0; Vga13h::WIDTH * Vga13h::HEIGHT],
			fb_b: [0; Vga13h::WIDTH * Vga13h::HEIGHT],
			show_fb_a: true,
		}
	}

	unsafe fn segment(&mut self) -> &'static mut [u8] {
		slice::from_raw_parts_mut(Vga13h::ADDR as *mut u8, Vga13h::WIDTH * Vga13h::HEIGHT)
	}

	pub fn swap(&mut self) {
		let seg = unsafe { self.segment() };
		if self.show_fb_a {
			seg.copy_from_slice(&self.fb_b);
			self.show_fb_a = false
		} else {
			seg.copy_from_slice(&self.fb_a);
			self.show_fb_a = true
		}
	}

	pub fn get_fb(&mut self) -> &mut [u8; Vga13h::WIDTH * Vga13h::HEIGHT] {
		unsafe { self.segment().try_into().unwrap() }

		/*
		if self.show_fb_a {
			&mut self.fb_b
		} else {
			&mut self.fb_a
		}
		*/
	}

	pub fn pix_idx(x: usize, y: usize) -> usize {
		debug_assert!(x < Vga13h::WIDTH);
		debug_assert!(y < Vga13h::HEIGHT);
		return y * Vga13h::WIDTH + x;
	}
}
